package Plugins::AlarmClock;

use Data::Dumper;
use strict;

###############################################################################
sub init {
    main::plugin_register("AlarmClock", "Alarm Clock", "/AlarmClock", \&Plugins::AlarmClock::html);
    main::add_timeout (time()+10, \&Plugins::AlarmClock::timer);
    main::http_register_handler("/AlarmClock/del",  \&Plugins::AlarmClock::del);
    main::http_register_handler("/AlarmClock/edit", \&Plugins::AlarmClock::edit);
    main::http_register_handler("/AlarmClock/index", \&Plugins::AlarmClock::index);

    delete $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{new};
}

###############################################################################
sub quit {
    delete $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{new};
}

###############################################################################
sub build_data {
    my ($id) = @_;

    my %row_data;

    my $alarm = \%{$main::PLUGIN_PREFS{AlarmClock}->{alarms}->{$id}};

    $row_data{ALARMCLOCK_INFO} = $alarm->{info};
    $row_data{ALARMCLOCK_NAME} = $alarm->{name};
    $row_data{ALARMCLOCK_TIME} = $alarm->{time};
    $row_data{ALARMCLOCK_BVOLUME} = $alarm->{bvolume};
    $row_data{ALARMCLOCK_EVOLUME} = $alarm->{evolume};
    $row_data{ALARMCLOCK_TVOLUME} = $alarm->{tvolume};
    $row_data{ALARMCLOCK_PLAYLIST} = $alarm->{playlist};
    $row_data{ALARMCLOCK_ID} = $id;
    $row_data{ALARMCLOCK_ENABLED} = "checked" if ($alarm->{enabled});
    $row_data{ALARMCLOCK_UNMUTE} = "checked" if ($alarm->{unmute});
    $row_data{ALARMCLOCK_ZONE} = $alarm->{zone};
    $row_data{ALARMCLOCK_ZONENAME} = $main::ZONES{$alarm->{zone}}->{ZoneName} if ($main::ZONES{$alarm->{zone}});

    my $zonemenu = "";
    foreach my $zone (keys %main::ZONES) {
        if ($zone eq $alarm->{zone}) {
            $zonemenu .= "<OPTION VALUE=$zone SELECTED>".$main::ZONES{$zone}->{ZoneName}."\n";
        } else {
            $zonemenu .= "<OPTION VALUE=$zone>".$main::ZONES{$zone}->{ZoneName}."\n";
        }
    }
    $row_data{ALARMCLOCK_ZONEMENU} = $zonemenu;

    my $days = "";
    foreach my $d ("sun", "mon", "tue", "wed", "thu", "fri", "sat") {
        if ($alarm->{$d}) {
            $row_data{"ALARMCLOCK_".uc($d)} = "checked" if ($alarm->{$d});
            $days .= ucfirst($d) . " ";
        }
    }
    $row_data{ALARMCLOCK_DAYS} = $days;

    return %row_data;
}

###############################################################################
sub html {
    my ($c, $r, $diskpath, $template) = @_;

    my %qf = $r->uri->query_form;

    my $now_string = localtime;
    $template->param("ALARMCLOCK_CURTIME" => $now_string);

    if ($template->query(name => "ALARMCLOCK_LOOP")) {
        my @loop_data = ();

        foreach my $id (sort (keys %{$main::PLUGIN_PREFS{AlarmClock}->{alarms}})) {
            next if ($id eq "new");
            my %row_data = build_data ($id);
            push(@loop_data, \%row_data);
        }

        $template->param("ALARMCLOCK_LOOP" => \@loop_data);
    }

    if (defined $qf{alarmclockid}) {
        if (defined $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{$qf{alarmclockid}}) {
            my %data = build_data($qf{alarmclockid});
            $template->param(%data);
        } else {
            $template->param("ALARMCLOCK_BVOLUME" => 10,
                             "ALARMCLOCK_EVOLUME" => 60,
                             "ALARMCLOCK_TVOLUME" => 5,
                             "ALARMCLOCK_ID"      => "new",
                             "ALARMCLOCK_TIME"    => "07:00",
                             "ALARMCLOCK_UNMUTE"  => "checked",
                             "ALARMCLOCK_MON"     => "checked",
                             "ALARMCLOCK_TUE"     => "checked",
                             "ALARMCLOCK_WED"     => "checked",
                             "ALARMCLOCK_THU"     => "checked",
                             "ALARMCLOCK_FRI"     => "checked");
            my $zonemenu = "";
            foreach my $zone (keys %main::ZONES) {
                $zonemenu .= "<OPTION VALUE=$zone>".$main::ZONES{$zone}->{ZoneName}."\n";
            }
            $template->param("ALARMCLOCK_ZONEMENU"=> $zonemenu);
        }
    }
}

###############################################################################
sub timer {
    main::add_timeout (time()+1, \&Plugins::AlarmClock::timer);

    my @days = ("sun", "mon", "tue", "wed", "thu", "fri", "sat");

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime time();
    foreach my $id (sort (keys %{$main::PLUGIN_PREFS{AlarmClock}->{alarms}})) {
        next if ($id eq "new");
        my $alarm = \%{$main::PLUGIN_PREFS{AlarmClock}->{alarms}->{$id}};

        if ($alarm->{running}) {
            # This alarm is currently running, see if we should bump volume
            if ($alarm->{last} + ($alarm->{tvolume}*60) <= time()) {
                $alarm->{running} = 0;
                main::upnp_render_volume($alarm->{zone}, $alarm->{evolume}); # Make sure we hit end volume
                next;
            }

            my $vol = int($alarm->{bvolume} +
                   ($alarm->{evolume} - $alarm->{bvolume}) * (time() - $alarm->{last})/($alarm->{tvolume}*60));

            if (main::upnp_render_volume($alarm->{zone}) != $vol) {
                main::upnp_render_volume($alarm->{zone}, $vol);
            }
        } else {
            # See if we should start this alarm
            next if (!$alarm->{enabled});
            next if ($alarm->{last} && (time() - $alarm->{last} < 120));
            next if ($alarm->{hour} != $hour);
            next if ($alarm->{min} != $min);
            next if (!$alarm->{$days[$wday]});

            main::Log(1, "Doing AlarmClock " . $alarm->{name});

            $alarm->{last} = time();
            $alarm->{running} = 1;

            if ($alarm->{unmute}) {
                main::upnp_render_mute($alarm->{zone}, 0);
            }

            main::upnp_render_volume($alarm->{zone}, $alarm->{bvolume});
            if ($alarm->{playlist}) {
                main::upnp_avtransport_action($alarm->{zone}, "RemoveAllTracksFromQueue");
                main::sonos_avtransport_add($alarm->{zone}, "/#Playlists/" . $alarm->{playlist});
            }
            main::upnp_avtransport_play($alarm->{zone});
        }
    }
}

###############################################################################
sub index {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;

    if (defined $qf{New}) {
        delete $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{new};
        $c->send_redirect(main::http_base_url($r) . "/AlarmClock/edit.html?alarmclockid=new", HTTP::Status::RC_MOVED_TEMPORARILY);
        $c->force_last_request;
        return;
    }

    foreach my $id (sort (keys %{$main::PLUGIN_PREFS{AlarmClock}->{alarms}})) {
        next if ($id eq "new");
        my $alarm = \%{$main::PLUGIN_PREFS{AlarmClock}->{alarms}->{$id}};
        $alarm->{enabled} = (defined $qf{"al$id"});
    }

    main::plugin_save_prefs();
    $c->send_redirect(main::http_base_url($r) . "/AlarmClock/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
    return;

}

###############################################################################
sub edit {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;

    if (defined $qf{Cancel}) {
        delete $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{new};
        $c->send_redirect(main::http_base_url($r) . "/AlarmClock/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
        $c->force_last_request;
        return;
    }

    my $id = $qf{id};
    my $alarm = \%{$main::PLUGIN_PREFS{AlarmClock}->{alarms}->{$id}};

    delete $alarm->{info};
    delete $alarm->{last};
    delete $alarm->{running};

    $alarm->{name} = $qf{name};
    $alarm->{info} = "<B>Must set a name</B>" if (length($alarm->{name}) == 0);
    $alarm->{time} = $qf{time};
    $alarm->{info} = "<B>Must set a time</B>" if (length($alarm->{time}) != 5);
    $alarm->{info} = "<B>Time wrong format, must be 5 characters 24hr time.</B>" if (!($alarm->{time} =~ /^[012][0-9]:[0-5][0-9]$/));
    $alarm->{hour} = int(substr($alarm->{time}, 0, 2));
    $alarm->{min}  = int(substr($alarm->{time}, 3, 2));

    foreach my $v ("bvolume", "evolume", "tvolume") {
        $alarm->{$v} = int($qf{$v});
        $alarm->{$v} = 0 if ($alarm->{$v} < 0);
        $alarm->{$v} = 100 if ($alarm->{$v} > 100);
    }
    $alarm->{playlist} = $qf{playlist};
    $alarm->{zone}     = $qf{zone};

    foreach my $d ("sun", "mon", "tue", "wed", "thu", "fri", "sat") {
        $alarm->{$d} = (defined $qf{$d});
    }
    $alarm->{unmute} = (defined $qf{unmute});

    if ($alarm->{info}) {
        $c->send_redirect(main::http_base_url($r) . "/AlarmClock/edit.html?alarmclockid=$id", HTTP::Status::RC_MOVED_TEMPORARILY);
        $c->force_last_request;
        return;
    }
    

    if ($id eq "new") {
        $main::PLUGIN_PREFS{AlarmClock}->{maxid}++;
        $id = $main::PLUGIN_PREFS{AlarmClock}->{maxid};
        $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{$id} = $alarm;
    }
    delete $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{new};
    main::plugin_save_prefs();

    $c->send_redirect(main::http_base_url($r) . "/AlarmClock/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
    return;

}

###############################################################################
sub del {
    my ($c, $r) = @_;

    my %qf = $r->uri->query_form;

    my $id = $qf{alarmclockid};
    delete $main::PLUGIN_PREFS{AlarmClock}->{alarms}->{$id};

    main::plugin_save_prefs();
    $c->send_redirect(main::http_base_url($r) . "/AlarmClock/index.html", HTTP::Status::RC_MOVED_TEMPORARILY);
    $c->force_last_request;
    return;

}

1;
